@@ -1,7 +1,7 @@
 /**************************************************************************
  *   help.c  --  This file is part of GNU nano.                           *
  *                                                                        *
- *   Copyright (C) 2000-2011, 2013-2021 Free Software Foundation, Inc.    *
+ *   Copyright (C) 2000-2011, 2013-2024 Free Software Foundation, Inc.    *
  *   Copyright (C) 2017 Rishabh Dave                                      *
  *   Copyright (C) 2014-2019 Benno Schulenberg                            *
  *                                                                        *
@@ -50,7 +50,7 @@ void help_init(void)
 	char *ptr;
 
 	/* First, set up the initial help text for the current function. */
-	if (currmenu & (MWHEREIS|MREPLACE|MREPLACEWITH)) {
+	if (currmenu & (MWHEREIS|MREPLACE)) {
 		htx[0] = N_("Search Command Help Text\n\n "
 				"Enter the words or characters you would like to "
 				"search for, and then press Enter.  If there is a "
@@ -65,6 +65,13 @@ void help_init(void)
 				"will be replaced.\n\n The following function keys are "
 				"available in Search mode:\n\n");
 		htx[2] = NULL;
+	} else if (currmenu == MREPLACEWITH) {
+		htx[0] = N_("=== Replacement ===\n\n "
+				"Type the characters that should replace what you "
+				"typed at the previous prompt, and press Enter.\n\n");
+		htx[1] = N_(" The following function keys "
+				"are available at this prompt:\n\n");
+		htx[2] = NULL;
 	} else if (currmenu == MGOTOLINE) {
 		htx[0] = N_("Go To Line Help Text\n\n "
 				"Enter the line number that you wish to go to and hit "
@@ -126,8 +133,8 @@ void help_init(void)
 				"shown in brackets after the search prompt.  Hitting "
 				"Enter without entering any text will perform the "
 				"previous search.\n\n");
-		htx[1] = N_(" The following function keys are available in "
-				"Browser Search mode:\n\n");
+		htx[1] = N_(" The following function keys "
+				"are available at this prompt:\n\n");
 		htx[2] = NULL;
 	} else if (currmenu == MGOTODIR) {
 		htx[0] = N_("Browser Go To Directory Help Text\n\n "
@@ -167,8 +174,8 @@ void help_init(void)
 		htx[1] = N_("If you just need another blank buffer, do not enter any "
 				"command.\n\n You can also pick one of four tools, or cut a "
 				"large piece of the buffer, or put the editor to sleep.\n\n");
-		htx[2] = N_(" The following function keys are "
-				"available in Execute Command mode:\n\n");
+		htx[2] = N_(" The following function keys "
+				"are available at this prompt:\n\n");
 	} else if (currmenu == MLINTER) {
 		htx[0] = N_("=== Linter ===\n\n "
 				"In this mode, the status bar shows an error message or "
@@ -224,7 +231,7 @@ void help_init(void)
 	 * plus translated text, plus one or two \n's. */
 	for (f = allfuncs; f != NULL; f = f->next)
 		if (f->menus & currmenu)
-			allocsize += strlen(_(f->help)) + 21;
+			allocsize += strlen(_(f->phrase)) + 21;
 
 #ifndef NANO_TINY
 	/* If we're on the main list, we also count the toggle help text.
@@ -234,8 +241,8 @@ void help_init(void)
 		size_t onoff_len = strlen(_("enable/disable"));
 
 		for (s = sclist; s != NULL; s = s->next)
-			if (s->func == do_toggle_void)
-				allocsize += strlen(_(flagtostr(s->toggle))) + onoff_len + 9;
+			if (s->func == do_toggle)
+				allocsize += strlen(_(epithet_of_flag(s->toggle))) + onoff_len + 9;
 	}
 #endif
 
@@ -282,7 +289,7 @@ void help_init(void)
 			ptr += 10;
 
 		/* The shortcut's description. */
-		ptr += sprintf(ptr, "%s\n", _(f->help));
+		ptr += sprintf(ptr, "%s\n", _(f->phrase));
 
 		if (f->blank_after)
 			ptr += sprintf(ptr, "\n");
@@ -303,8 +310,9 @@ void help_init(void)
 			for (s = sclist; s != NULL; s = s->next)
 				if (s->toggle && s->ordinal == counter) {
 					ptr += sprintf(ptr, "%s\t\t %s %s\n", (s->menus & MMAIN ? s->keystr : ""),
-								_(flagtostr(s->toggle)), _("enable/disable"));
-					if (s->toggle == NO_SYNTAX || s->toggle == TABS_TO_SPACES)
+								_(epithet_of_flag(s->toggle)), _("enable/disable"));
+					/* Add a blank like between two groups. */
+					if (s->toggle == NO_SYNTAX)
 						ptr += sprintf(ptr, "\n");
 					break;
 				}
@@ -316,20 +324,27 @@ void help_init(void)
 /* Hard-wrap the concatenated help text, and write it into a new buffer. */
 void wrap_help_text_into_buffer(void)
 {
-	size_t sum = 0;
 	/* Avoid overtight and overwide paragraphs in the introductory text. */
-	size_t wrapping_point = ((COLS < 40) ? 40 : (COLS > 74) ? 74 : COLS) - thebar;
+	size_t wrapping_point = ((COLS < 40) ? 40 : (COLS > 74) ? 74 : COLS) - sidebar;
 	const char *ptr = start_of_body;
+	size_t sum = 0;
 
 	make_new_buffer();
 
+	/* Ensure there is a blank line at the top of the text, for esthetics. */
+	if ((ISSET(MINIBAR) || !ISSET(EMPTY_LINE)) && LINES > 6) {
+		openfile->current->data = mallocstrcpy(openfile->current->data, " ");
+		openfile->current->next = make_new_node(openfile->current);
+		openfile->current = openfile->current->next;
+	}
+
 	/* Copy the help text into the just-created new buffer. */
 	while (*ptr != '\0') {
 		int length, shim;
 		char *oneline;
 
 		if (ptr == end_of_intro)
-			wrapping_point = ((COLS < 40) ? 40 : COLS) - thebar;
+			wrapping_point = ((COLS < 40) ? 40 : COLS) - sidebar;
 
 		if (ptr < end_of_intro || *(ptr - 1) == '\n') {
 			length = break_line(ptr, wrapping_point, TRUE);
@@ -337,7 +352,7 @@ void wrap_help_text_into_buffer(void)
 			shim = (*(ptr + length - 1) == ' ') ? 0 : 1;
 			snprintf(oneline, length + shim, "%s", ptr);
 		} else {
-			length = break_line(ptr, ((COLS < 40) ? 22 : COLS - 18) - thebar, TRUE);
+			length = break_line(ptr, ((COLS < 40) ? 22 : COLS - 18) - sidebar, TRUE);
 			oneline = nmalloc(length + 5);
 			snprintf(oneline, length + 5, "\t\t  %s", ptr);
 		}
@@ -381,7 +396,7 @@ void wrap_help_text_into_buffer(void)
 void show_help(void)
 {
 	int kbinput = ERR;
-	functionptrtype func;
+	functionptrtype function;
 		/* The function of the key the user typed in. */
 	int oldmenu = currmenu;
 		/* The menu we were called from. */
@@ -403,8 +418,9 @@ void show_help(void)
 	memcpy(stash, flags, sizeof(flags));
 
 	/* Ensure that the help screen's shortcut list can be displayed. */
-	if (ISSET(NO_HELP) && LINES > 4) {
+	if (ISSET(NO_HELP) || ISSET(ZERO)) {
 		UNSET(NO_HELP);
+		UNSET(ZERO);
 		window_init();
 	} else
 		blank_statusbar();
@@ -415,11 +431,9 @@ void show_help(void)
 	UNSET(USE_REGEXP);
 
 	UNSET(WHITESPACE_DISPLAY);
-	UNSET(NOREAD_MODE);
 
 #ifdef ENABLE_LINENUMBERS
-	UNSET(LINE_NUMBERS);
-	editwincols = COLS - thebar;
+	editwincols = COLS - sidebar;
 	margin = 0;
 #endif
 	tabsize = 8;
@@ -456,41 +470,40 @@ void show_help(void)
 		focusing = TRUE;
 
 		/* Show the cursor when we searched and found something. */
-		kbinput = get_kbinput(edit, didfind == 1 || ISSET(SHOW_CURSOR));
+		kbinput = get_kbinput(midwin, didfind == 1 || ISSET(SHOW_CURSOR));
 
 		didfind = 0;
 
 #ifndef NANO_TINY
 		spotlighted = FALSE;
-		hide_cursor = FALSE;
 
 		if (bracketed_paste || kbinput == BRACKETED_PASTE_MARKER) {
 			beep();
 			continue;
 		}
 #endif
-		func = interpret(&kbinput);
+		function = interpret(kbinput);
 
-		if (func == full_refresh) {
+		if (function == full_refresh) {
 			full_refresh();
-		} else if (ISSET(SHOW_CURSOR) && (func == do_left || func == do_right ||
-											func == do_up || func == do_down)) {
-			func();
-		} else if (func == do_up || func == do_scroll_up) {
+		} else if (ISSET(SHOW_CURSOR) && (function == do_left || function == do_right ||
+											function == do_up || function == do_down)) {
+			function();
+		} else if (function == do_up || function == do_scroll_up) {
 			do_scroll_up();
-		} else if (func == do_down || func == do_scroll_down) {
+		} else if (function == do_down || function == do_scroll_down) {
 			if (openfile->edittop->lineno + editwinrows - 1 < openfile->filebot->lineno)
 				do_scroll_down();
-		} else if (func == do_page_up || func == do_page_down ||
-					func == to_first_line || func == to_last_line) {
-			func();
-		} else if (func == do_search_backward || func == do_search_forward ||
-					func == do_findprevious || func == do_findnext) {
-			func();
+		} else if (function == do_page_up || function == do_page_down ||
+					function == to_first_line || function == to_last_line) {
+			function();
+		} else if (function == do_search_backward || function == do_search_forward ||
+					function == do_findprevious || function == do_findnext) {
+			function();
 			bottombars(MHELP);
 #ifdef ENABLE_NANORC
-		} else if (func == (functionptrtype)implant) {
-			implant(first_sc_for(MHELP, func)->expansion);
+		} else if (function == (functionptrtype)implant) {
+			implant(first_sc_for(MHELP, function)->expansion);
 #endif
 #ifdef ENABLE_MOUSE
 		} else if (kbinput == KEY_MOUSE) {
@@ -501,7 +514,7 @@ void show_help(void)
 		} else if (kbinput == KEY_WINCH) {
 			;  /* Nothing to do. */
 #endif
-		} else if (func == do_exit) {
+		} else if (function == do_exit) {
 			break;
 		} else
 			unbound_key(kbinput);
@@ -526,7 +539,7 @@ void show_help(void)
 
 #ifdef ENABLE_LINENUMBERS
 	margin = was_margin;
-	editwincols = COLS - margin - thebar;
+	editwincols = COLS - margin - sidebar;
 #endif
 	tabsize = was_tabsize;
 #ifdef ENABLE_COLOR
@@ -543,13 +556,12 @@ void show_help(void)
 
 	curs_set(0);
 
-	if (ISSET(NO_HELP)) {
-		currmenu = oldmenu;
+	if (ISSET(NO_HELP) || ISSET(ZERO))
 		window_init();
-	} else {
+	else
 		blank_statusbar();
-		bottombars(oldmenu);
-	}
+
+	bottombars(oldmenu);
 
 #ifdef ENABLE_BROWSER
 	if (oldmenu & (MBROWSER|MWHEREISFILE|MGOTODIR))
