@@ -1,7 +1,7 @@
 /**************************************************************************
  *   rcfile.c  --  This file is part of GNU nano.                         *
  *                                                                        *
- *   Copyright (C) 2001-2011, 2013-2022 Free Software Foundation, Inc.    *
+ *   Copyright (C) 2001-2011, 2013-2024 Free Software Foundation, Inc.    *
  *   Copyright (C) 2014 Mike Frysinger                                    *
  *   Copyright (C) 2019 Brand Huntsman                                    *
  *   Copyright (C) 2014-2021 Benno Schulenberg                            *
@@ -153,18 +153,6 @@ static bool seen_color_command = FALSE;
 static colortype *lastcolor = NULL;
 		/* The end of the color list for the current syntax. */
 #endif
-
-#define NUMBER_OF_MENUS  16
-char *menunames[NUMBER_OF_MENUS] = { "main", "search", "replace", "replacewith",
-									"yesno", "gotoline", "writeout", "insert",
-									"execute", "help", "spell", "linter",
-									"browser", "whereisfile", "gotodir",
-									"all" };
-int menusymbols[NUMBER_OF_MENUS] = { MMAIN, MWHEREIS, MREPLACE, MREPLACEWITH,
-									MYESNO, MGOTOLINE, MWRITEFILE, MINSERTFILE,
-									MEXECUTE, MHELP, MSPELL, MLINTER,
-									MBROWSER, MWHEREISFILE, MGOTODIR,
-									MMOST|MBROWSER|MHELP|MYESNO };
 #endif /* ENABLE_NANORC */
 
 #if defined(ENABLE_NANORC) || defined(ENABLE_HISTORIES)
@@ -257,6 +245,8 @@ keystruct *strtosc(const char *input)
 		s->func = do_replace;
 	else if (!strcmp(input, "cut"))
 		s->func = cut_text;
+	else if (!strcmp(input, "copy"))
+		s->func = copy_text;
 	else if (!strcmp(input, "paste"))
 		s->func = paste_text;
 #ifndef NANO_TINY
@@ -264,8 +254,6 @@ keystruct *strtosc(const char *input)
 		s->func = do_execute;
 	else if (!strcmp(input, "cutrestoffile"))
 		s->func = cut_till_eof;
-	else if (!strcmp(input, "copy"))
-		s->func = copy_text;
 	else if (!strcmp(input, "zap"))
 		s->func = zap_text;
 	else if (!strcmp(input, "mark"))
@@ -276,9 +264,11 @@ keystruct *strtosc(const char *input)
 	         !strcmp(input, "speller"))
 		s->func = do_spell;
 #endif
-#ifdef ENABLE_COLOR
+#ifdef ENABLE_LINTER
 	else if (!strcmp(input, "linter"))
 		s->func = do_linter;
+#endif
+#ifdef ENABLE_FORMATTER
 	else if (!strcmp(input, "formatter"))
 		s->func = do_formatter;
 #endif
@@ -364,6 +354,12 @@ keystruct *strtosc(const char *input)
 		s->func = to_prev_block;
 	else if (!strcmp(input, "nextblock"))
 		s->func = to_next_block;
+#ifndef NANO_TINY
+	else if (!strcmp(input, "toprow"))
+		s->func = to_top_row;
+	else if (!strcmp(input, "bottomrow"))
+		s->func = to_bottom_row;
+#endif
 	else if (!strcmp(input, "pageup") ||
 	         !strcmp(input, "prevpage"))
 		s->func = do_page_up;
@@ -493,6 +489,18 @@ keystruct *strtosc(const char *input)
 	return s;
 }
 
+#define NUMBER_OF_MENUS  16
+char *menunames[NUMBER_OF_MENUS] = { "main", "search", "replace", "replacewith",
+									"yesno", "gotoline", "writeout", "insert",
+									"execute", "help", "spell", "linter",
+									"browser", "whereisfile", "gotodir",
+									"all" };
+int menusymbols[NUMBER_OF_MENUS] = { MMAIN, MWHEREIS, MREPLACE, MREPLACEWITH,
+									MYESNO, MGOTOLINE, MWRITEFILE, MINSERTFILE,
+									MEXECUTE, MHELP, MSPELL, MLINTER,
+									MBROWSER, MWHEREISFILE, MGOTODIR,
+									MMOST|MBROWSER|MHELP|MYESNO };
+
 /* Return the symbol that corresponds to the given menu name. */
 int name_to_menu(const char *name)
 {
@@ -502,7 +510,7 @@ int name_to_menu(const char *name)
 		if (strcmp(name, menunames[index]) == 0)
 			return menusymbols[index];
 
-	return -1;
+	return 0;
 }
 
 /* Return the name that corresponds to the given menu symbol. */
@@ -672,12 +680,12 @@ void begin_new_syntax(char *ptr)
 	live_syntax->magics = NULL;
 	live_syntax->linter = NULL;
 	live_syntax->formatter = NULL;
-	live_syntax->tab = NULL;
+	live_syntax->tabstring = NULL;
 #ifdef ENABLE_COMMENT
 	live_syntax->comment = copy_of(GENERAL_COMMENT_CHARACTER);
 #endif
 	live_syntax->color = NULL;
-	live_syntax->nmultis = 0;
+	live_syntax->multiscore = 0;
 
 	/* Hook the new syntax in at the top of the list. */
 	live_syntax->next = syntaxes;
@@ -785,6 +793,12 @@ void parse_binding(char *ptr, bool dobind)
 		goto free_things;
 	}
 
+	menu = name_to_menu(menuptr);
+	if (menu == 0) {
+		jot_error(N_("Unknown menu: %s"), menuptr);
+		goto free_things;
+	}
+
 	if (dobind) {
 		/* If the thing to bind starts with a double quote, it is a string,
 		 * otherwise it is the name of a function. */
@@ -799,17 +813,11 @@ void parse_binding(char *ptr, bool dobind)
 			newsc = strtosc(funcptr);
 
 		if (newsc == NULL) {
-			jot_error(N_("Cannot map name \"%s\" to a function"), funcptr);
+			jot_error(N_("Unknown function: %s"), funcptr);
 			goto free_things;
 		}
 	}
 
-	menu = name_to_menu(menuptr);
-	if (menu < 1) {
-		jot_error(N_("Cannot map name \"%s\" to a menu"), menuptr);
-		goto free_things;
-	}
-
 	/* Wipe the given shortcut from the given menu. */
 	for (keystruct *s = sclist; s != NULL; s = s->next)
 		if ((s->menus & menu) && s->keycode == keycode)
@@ -906,7 +914,7 @@ void parse_one_include(char *file, syntaxtype *syntax)
 	FILE *rcstream;
 
 	/* Don't open directories, character files, or block files. */
-	if (!is_good_file(file))
+	if (access(file, R_OK) == 0 && !is_good_file(file))
 		return;
 
 	rcstream = fopen(file, "rb");
@@ -972,9 +980,14 @@ void parse_includes(char *ptr)
 		pattern++;
 	ptr = parse_argument(ptr);
 
+	if (strlen(pattern) > PATH_MAX) {
+		jot_error(N_("Path is too long"));
+		return;
+	}
+
 	/* Expand a tilde first, then try to match the globbing pattern. */
 	expanded = real_dir_from_tilde(pattern);
-	result = glob(expanded, GLOB_ERR, NULL, &files);
+	result = glob(expanded, GLOB_ERR|GLOB_NOCHECK, NULL, &files);
 
 	/* If there are matches, process each of them.  Otherwise, only
 	 * report an error if it's something other than zero matches. */
@@ -989,16 +1002,22 @@ void parse_includes(char *ptr)
 }
 
 /* Return the index of the color that is closest to the given RGB levels,
- * assuming that the terminal uses the 6x6x6 color cube of xterm-256color. */
+ * assuming that the terminal uses the 6x6x6 color cube of xterm-256color.
+ * When red == green == blue, return an index in the xterm gray scale. */
 short closest_index_color(short red, short green, short blue)
 {
-	/* Translation table, from 16 intended levels to 6 available levels. */
+	/* Translation table, from 16 intended color levels to 6 available levels. */
 	static const short level[] = { 0, 0, 0, 0, 1, 1, 1, 1, 2, 2, 3, 3, 4, 4, 5, 5 };
 
-	if (COLORS == 256)
-		return (36 * level[red] + 6 * level[green] + level[blue] + 16);
-	else
+	/* Translation table, from 14 intended gray levels to 24 available levels. */
+	static const short gray[] = { 1, 2, 3, 4, 5, 6, 7, 9, 11, 13, 15, 18, 21, 23 };
+
+	if (COLORS != 256)
 		return THE_DEFAULT;
+	else if (red == green && green == blue && 0 < red && red < 0xF)
+		return 232 + gray[red - 1];
+	else
+		return (36 * level[red] + 6 * level[green] + level[blue] + 16);
 }
 
 #define COLORCOUNT  34
@@ -1071,40 +1090,40 @@ short color_to_short(const char *colorname, bool *vivid, bool *thick)
 
 /* Parse the color name (or pair of color names) in the given string.
  * Return FALSE when any color name is invalid; otherwise return TRUE. */
-bool parse_combination(char *combostr, short *fg, short *bg, int *attributes)
+bool parse_combination(char *combotext, short *fg, short *bg, int *attributes)
 {
 	bool vivid, thick;
 	char *comma;
 
 	*attributes = A_NORMAL;
 
-	if (strncmp(combostr, "bold", 4) == 0) {
+	if (strncmp(combotext, "bold", 4) == 0) {
 		*attributes |= A_BOLD;
-		if (combostr[4] != ',') {
+		if (combotext[4] != ',') {
 			jot_error(N_("An attribute requires a subsequent comma"));
 			return FALSE;
 		}
-		combostr += 5;
+		combotext += 5;
 	}
 
-	if (strncmp(combostr, "italic", 6) == 0) {
+	if (strncmp(combotext, "italic", 6) == 0) {
 #ifdef A_ITALIC
 		*attributes |= A_ITALIC;
 #endif
-		if (combostr[6] != ',') {
+		if (combotext[6] != ',') {
 			jot_error(N_("An attribute requires a subsequent comma"));
 			return FALSE;
 		}
-		combostr += 7;
+		combotext += 7;
 	}
 
-	comma = strchr(combostr, ',');
+	comma = strchr(combotext, ',');
 
 	if (comma)
 		*comma = '\0';
 
-	if (!comma || comma > combostr) {
-		*fg = color_to_short(combostr, &vivid, &thick);
+	if (!comma || comma > combotext) {
+		*fg = color_to_short(combotext, &vivid, &thick);
 		if (*fg == BAD_COLOR)
 			return FALSE;
 		if (vivid && !thick && COLORS > 8)
@@ -1210,22 +1229,22 @@ void parse_rule(char *ptr, int rex_flags)
 
 		/* For a multiline rule, give it a number and increase the count. */
 		if (expectend) {
-			newcolor->id = live_syntax->nmultis;
-			live_syntax->nmultis++;
+			newcolor->id = live_syntax->multiscore;
+			live_syntax->multiscore++;
 		}
 	}
 }
 
-/* Parse the argument of an interface color option. */
-colortype *parse_interface_color(char *combostr)
+/* Set the colors for the given interface element to the given combination. */
+void set_interface_color(int element, char *combotext)
 {
 	colortype *trio = nmalloc(sizeof(colortype));
 
-	if (!parse_combination(combostr, &trio->fg, &trio->bg, &trio->attributes)) {
-		free(trio);
-		return NULL;
+	if (parse_combination(combotext, &trio->fg, &trio->bg, &trio->attributes)) {
+		free(color_combo[element]);
+		color_combo[element] = trio;
 	} else
-		return trio;
+		free(trio);
 }
 
 /* Read regex strings enclosed in double quotes from the line pointed at
@@ -1323,12 +1342,14 @@ bool parse_syntax_commands(char *keyword, char *ptr)
 		pick_up_name("comment", ptr, &live_syntax->comment);
 #endif
 	} else if (strcmp(keyword, "tabgives") == 0) {
-		pick_up_name("tabgives", ptr, &live_syntax->tab);
-	} else if (strcmp(keyword, "linter") == 0)
+		pick_up_name("tabgives", ptr, &live_syntax->tabstring);
+	} else if (strcmp(keyword, "linter") == 0) {
 		pick_up_name("linter", ptr, &live_syntax->linter);
-	else if (strcmp(keyword, "formatter") == 0)
+		strip_leading_blanks_from(live_syntax->linter);
+	} else if (strcmp(keyword, "formatter") == 0) {
 		pick_up_name("formatter", ptr, &live_syntax->formatter);
-	else
+		strip_leading_blanks_from(live_syntax->formatter);
+	} else
 		return FALSE;
 
 	return TRUE;
@@ -1345,10 +1366,10 @@ static void check_vitals_mapped(void)
 
 	for (int v = 0; v < VITALS; v++) {
 		for (funcstruct *f = allfuncs; f != NULL; f = f->next) {
-			if (f->func == vitals[v] && f->menus & inmenus[v]) {
+			if (f->func == vitals[v] && (f->menus & inmenus[v])) {
 				if (first_sc_for(inmenus[v], f->func) == NULL) {
 					jot_error(N_("No key is bound to function '%s' in menu '%s'. "
-								" Exiting.\n"), f->desc, menu_to_name(inmenus[v]));
+								" Exiting.\n"), f->tag, menu_to_name(inmenus[v]));
 					die(_("If needed, use nano with the -I option "
 								"to adjust your nanorc settings.\n"));
 				} else
@@ -1530,7 +1551,7 @@ void parse_rcfile(FILE *rcstream, bool just_syntax, bool intros_only)
 		}
 
 		if (rcopts[i].name == NULL) {
-			jot_error(N_("Unknown option \"%s\""), option);
+			jot_error(N_("Unknown option: %s"), option);
 			continue;
 		}
 
@@ -1568,29 +1589,29 @@ void parse_rcfile(FILE *rcstream, bool just_syntax, bool intros_only)
 #endif
 #ifdef ENABLE_COLOR
 		if (strcmp(option, "titlecolor") == 0)
-			color_combo[TITLE_BAR] = parse_interface_color(argument);
+			set_interface_color(TITLE_BAR, argument);
 		else if (strcmp(option, "numbercolor") == 0)
-			color_combo[LINE_NUMBER] = parse_interface_color(argument);
+			set_interface_color(LINE_NUMBER, argument);
 		else if (strcmp(option, "stripecolor") == 0)
-			color_combo[GUIDE_STRIPE] = parse_interface_color(argument);
+			set_interface_color(GUIDE_STRIPE, argument);
 		else if (strcmp(option, "scrollercolor") == 0)
-			color_combo[SCROLL_BAR] = parse_interface_color(argument);
+			set_interface_color(SCROLL_BAR, argument);
 		else if (strcmp(option, "selectedcolor") == 0)
-			color_combo[SELECTED_TEXT] = parse_interface_color(argument);
+			set_interface_color(SELECTED_TEXT, argument);
 		else if (strcmp(option, "spotlightcolor") == 0)
-			color_combo[SPOTLIGHTED] = parse_interface_color(argument);
+			set_interface_color(SPOTLIGHTED, argument);
 		else if (strcmp(option, "minicolor") == 0)
-			color_combo[MINI_INFOBAR] = parse_interface_color(argument);
+			set_interface_color(MINI_INFOBAR, argument);
 		else if (strcmp(option, "promptcolor") == 0)
-			color_combo[PROMPT_BAR] = parse_interface_color(argument);
+			set_interface_color(PROMPT_BAR, argument);
 		else if (strcmp(option, "statuscolor") == 0)
-			color_combo[STATUS_BAR] = parse_interface_color(argument);
+			set_interface_color(STATUS_BAR, argument);
 		else if (strcmp(option, "errorcolor") == 0)
-			color_combo[ERROR_MESSAGE] = parse_interface_color(argument);
+			set_interface_color(ERROR_MESSAGE, argument);
 		else if (strcmp(option, "keycolor") == 0)
-			color_combo[KEY_COMBO] = parse_interface_color(argument);
+			set_interface_color(KEY_COMBO, argument);
 		else if (strcmp(option, "functioncolor") == 0)
-			color_combo[FUNCTION_TAG] = parse_interface_color(argument);
+			set_interface_color(FUNCTION_TAG, argument);
 		else
 #endif
 #ifdef ENABLE_OPERATINGDIR
@@ -1705,7 +1726,7 @@ void do_rcfiles(void)
 {
 	if (custom_nanorc) {
 		nanorc = get_full_path(custom_nanorc);
-		if (access(nanorc, F_OK) != 0)
+		if (nanorc == NULL || access(nanorc, F_OK) != 0)
 			die(_("Specified rcfile does not exist\n"));
 	} else
 		nanorc = mallocstrcpy(nanorc, SYSCONFDIR "/nanorc");
